# gobserver

[![Build status](https://gitlab.com/gLagiewka/gobserver/badges/master/build.svg)](https://gitlab.com/gLagiewka/gobserver/commits/master)
[![Coverage](https://gitlab.com/gLagiewka/gobserver/badges/master/coverage.svg)](https://gLagiewka.gitlab.io/gobserver/)
[![License](https://img.shields.io/github/license/mashape/apistatus.svg)](https://gitlab.com/gLagiewka/gobserver/blob/master/LICENSE.md)

A simple implementation of the observer pattern.

---

## Supported browsers

es5 compliant browsers.

---

## Installing

When using yarn:
```js
yarn add gobserver
```

When using npm:
```js
npm install gobserver
```

When using a script tag:
```html
<script src="gobserver.js"></script>
```
---

## Importing

When using TypeScript or es2015 modules:
```ts
import * as gobserver from "gobserver";

// or

import { create } from "gobserver";
```

When using CommonJS:
```ts
const gobserver = require("gobserver");
```

When using AMD:
```ts
define(["gobserver"], (gobserver) => {
  // ...
})
```
---

## Creating

When using JavaScipt:
```ts
const observer = gobserver.create();
```

When using TypeScript:
```ts
type Event = {
  data: any
};

const observer = gobserver.create<Event>();
```

When using a custom return value:
```ts
type Event = {
  data: any
};

class Service {
  public readonly onUpdate = gobserver.create<Event, Service>({ returnValue: this });
}
```
---

## Subscribing / Publishing

Subscribing:

```ts
observer.subscribe((evt: Event) => {
  console.log(evt.data);
});
```

Publishing:

```ts
observer.publish({ data: 12 });
```

Registering a handler that is executed only once:

```ts

observer.subscribe((evt: Event) => {
  console.log(evt.data); // This is executed only once
}, { once: true });

observer.publish({ data: 12 });
observer.publish({ data: 12 });
```

Registering a handler that is executed under the given context:
```js
const obj = {
  value: 1,
  foo(evt: Event) {
    console.log(this.value + evt.data);
  },
};

observer.subscribe(obj.foo, { context: obj })

observer.publish({ data: 2 });
```

Unsubscribing:

```ts
observer.unsubscribe(handler);
```
---

## Pipe / Unpipe

Pipe events from one observer to another:

```ts
const observer = gobserver.create<Event>();
const pipedObserver = gobserver.create<Event>();

pipedObserver.subscribe((evt: Event) => {
  console.log(evt.data);
});

observer.pipe(pipedObserver);

observer.publish({ data: 12 });
```

Unpipe both observers:

```ts
observer.unpipe(pipedObserver);
```
---

## Chaining

```ts
gobserver
  .create<Event>()
  .subscribe((evt: Event) => {
    console.log(evt.data);    
  })
  .publish({ data: 12 });
```

When using a custom return value:

```ts
class Service {
  public readonly onUpdate = gobserver.create<Event, Service>({ returnValue: this });

  public doSomething() {
    // ...

    return this;
  }
}

new Service()
  .onUpdate.subscribe((evt: any) => {
    console.log(evt.data);
  })
  .doSomething()
  .onUpdate.publish({ data: 12 });

```
---

## Clearing

Removing every registered handler and piped observer:

```ts
observer.clear();
```

---

## License

This project is licensed under the MIT License - see the [LICENSE.md](https://gitlab.com/gLagiewka/gobserver/blob/master/LICENSE.md) file for details.
