/**
 * Observer's handler function.
 */
export type ObserverHandler<TData> = (data: TData) => void;
/**
 * Options of the subscribe method.
 */
export type SubscribeOptions = {
  /**
   * If set to true, the given handler will be immediately unregistered, after it has been fired. Defaults to false.
   */
  once?: boolean;
  /**
   * The value of the context under which the given handler should be executed. Defaults to undefined.
   */
  context?: any;
};
/**
 * Observer's public interface.
 */
export interface Observer<TData> {
  /**
   * Registers the given handler.
   *
   * @param handler   A function that should be called when this observer emits.
   * @param options   Options.
   * @returns         This instance.
   */
  subscribe(handler: ObserverHandler<TData>, options?: SubscribeOptions): this;
  /**
   * Executes every handler with the given data.
   *
   * @param data  The data that will be passed to every handler.
   * @returns     This instance.
   */
  publish(data: TData): this;
  /**
   * Removes the given handler.
   *
   * @param handler   A handler that should be removed from this observer.
   * @returns         This instance.
   */
  unsubscribe(handler: ObserverHandler<TData>): this;
  /**
   * Removes every handler and unpipes every piped observer.
   *
   * @returns  This instance.
   */
  clear(): this;
  /**
   * Pipes events from this observer to the given one.
   *
   * @param observer  The observer which should be piped.
   * @returns         This instance.
   */
  pipe(observer: Observer<TData> | ObserverWithCustomReturn<TData, {}>): this;
  /**
   * Unpipes this observer from the given one.
   *
   * @param observer  The observer which should be unpiped.
   * @returns         This instance.
   */
  unpipe(observer: Observer<TData> | ObserverWithCustomReturn<TData, {}>): this;
}
/**
 * Observer's public interface.
 */
export interface ObserverWithCustomReturn<TData, TReturn> {
  /**
   * Registers the given handler.
   *
   * @param handler   A function that should be called when this observer emits.
   * @param options   Options.
   * @returns         The given return value.
   */
  subscribe(handler: ObserverHandler<TData>, options?: SubscribeOptions): TReturn;
  /**
   * Executes every handler with the given data.
   *
   * @param data  The data that will be passed to every handler.
   * @returns     The given return value.
   */
  publish(data: TData): TReturn;
  /**
   * Removes the given handler.
   *
   * @param handler   A handler that should be removed from this observer.
   * @returns         The given return value.
   */
  unsubscribe(handler: ObserverHandler<TData>): TReturn;
  /**
   * Removes every handler and unpipes every piped observer.
   *
   * @returns  The given return value.
   */
  clear(): TReturn;
  /**
   * Pipes events from this observer to the given one.
   *
   * @param observer  The observer which should be piped.
   * @returns         The given return value.
   */
  pipe(observer: Observer<TData> | ObserverWithCustomReturn<TData, TReturn>): TReturn;
  /**
   * Unpipes this observer from the given one.
   *
   * @param observer  The observer which should be unpiped.
   * @returns         The given return value.
   */
  unpipe(observer: Observer<TData> | ObserverWithCustomReturn<TData, TReturn>): TReturn;
}
/**
 * Creates an observer.
 *
 * @returns   An observer.
 */
export function create<TData>(): Observer<TData>;
export function create<TData, TReturn>({ returnValue }: {
  returnValue: TReturn;
}): ObserverWithCustomReturn<TData, TReturn>;
export as namespace gobserver;
